﻿using System;
using System.Web;
using System.Workflow.Runtime;
using System.Diagnostics;
using System.Workflow.Runtime.Hosting;
using WorkflowCore.Logging;
using System.Collections.Generic;
using System.Configuration;

//////////////////////////////////////////////////////////////////////
// Created by Michael Kennedy
// http://www.michaelckennedy.net
// Copyright Michael Kennedy 2008.
// Part of the HRWorkflow sample application featured in MSDN Magazine.
//

namespace HRWorkflow
{
	public class Global : HttpApplication
	{
		public static WorkflowRuntime WorkflowRuntime { get; set; }
		public static Dictionary<Guid, Stopwatch> timers = 
			new Dictionary<Guid, Stopwatch>();

		protected void Application_Start(object sender, EventArgs e)
		{
			WorkflowRuntime = new WorkflowRuntime();

			InstallPersistenceService();

			WorkflowRuntime.WorkflowCompleted += WorkflowRuntime_WorkflowCompleted;
			WorkflowRuntime.WorkflowTerminated += WorkflowRuntime_WorkflowTerminated;
			WorkflowRuntime.WorkflowIdled += WorkflowRuntime_WorkflowIdled;
			WorkflowRuntime.WorkflowResumed += WorkflowRuntime_WorkflowResumed;
			WorkflowRuntime.WorkflowStarted += WorkflowRuntime_WorkflowStarted;
			WorkflowRuntime.WorkflowPersisted += WorkflowRuntime_WorkflowPersisted;
			WorkflowRuntime.WorkflowLoaded += WorkflowRuntime_WorkflowLoaded;

			WorkflowRuntime.StartRuntime();

			TraceDT.Add( "Global: Creating and starting WorkflowRuntime.", 
				Guid.Empty, this.GetType() );
		}

		void WorkflowRuntime_WorkflowLoaded(object sender, WorkflowEventArgs e)
		{
			TraceDT.Add("Global: Workflow instance restored from database.",
				e.WorkflowInstance.InstanceId, this.GetType());
		}

		void WorkflowRuntime_WorkflowPersisted(object sender, WorkflowEventArgs e)
		{
			TraceDT.Add("Global: Workflow instance unloaded and persisted to database.",
				e.WorkflowInstance.InstanceId, this.GetType());
		}

		void WorkflowRuntime_WorkflowStarted(object sender, WorkflowEventArgs e)
		{
			TraceDT.Add( "Global: Workflow instance started.",
				e.WorkflowInstance.InstanceId, this.GetType() );

			Stopwatch watch = GetTimerByWorkflow( e.WorkflowInstance.InstanceId );
			watch.Start();
		}

		private void WorkflowRuntime_WorkflowResumed(object sender, WorkflowEventArgs e)
		{
			TraceDT.Add( "Global: Workflow instance resumed.",
				e.WorkflowInstance.InstanceId, this.GetType() );
			Stopwatch watch = GetTimerByWorkflow( e.WorkflowInstance.InstanceId );
			watch.Start();
		}

		private void WorkflowRuntime_WorkflowIdled(object sender, WorkflowEventArgs e)
		{
			Stopwatch watch = GetTimerByWorkflow( e.WorkflowInstance.InstanceId );
			watch.Stop();
			TraceDT.Add( "Global: Workflow instance idled.", 
				e.WorkflowInstance.InstanceId, this.GetType() );
		}

		private void WorkflowRuntime_WorkflowCompleted(object sender, WorkflowCompletedEventArgs e)
		{
			Stopwatch watch = GetTimerByWorkflow( e.WorkflowInstance.InstanceId );
			watch.Stop();
			TraceDT.Add( "Global: Workflow instance finished with response {0}. Total 'active' time is {1:N3} seconds.", 
				e.WorkflowInstance.InstanceId, 
				this.GetType(), 
				e.OutputParameters["EmployeeResponse"], 
				watch.ElapsedMilliseconds / 1000.0 );
		}

		void WorkflowRuntime_WorkflowTerminated(object sender, WorkflowTerminatedEventArgs e)
		{
			TraceDT.Add("Global: Workflow has crashed: " + e.Exception.Message,
				e.WorkflowInstance.InstanceId, this.GetType());
		}

		protected void Session_Start(object sender, EventArgs e)
		{
		}

		protected void Application_BeginRequest(object sender, EventArgs e)
		{
		}

		protected void Application_AuthenticateRequest(object sender, EventArgs e)
		{
		}

		protected void Application_Error(object sender, EventArgs e)
		{
		}

		protected void Session_End(object sender, EventArgs e)
		{
		}

		protected void Application_End(object sender, EventArgs e)
		{
			TraceDT.Add( "Global: Shutting down WorkflowRuntime...",
				Guid.Empty, this.GetType() );
			WorkflowRuntime.StopRuntime();
			WorkflowRuntime.Dispose();
		}

		private void InstallPersistenceService()
		{
			//
			// Almost certainly would want persistence for spanning worker 
			// process restarts if this were a 'real' application.
			//

			//
			// Run the scripts to create the sql persistence DB at
			//    C:\Windows\Microsoft.NET\Framework\v3.0\Windows Workflow Foundation\SQL\EN
			//      AND
			//    C:\Windows\Microsoft.NET\Framework\v3.5\SQL\EN
			//

			if (!GetUseDB())
            {
				return;
            }

			ConnectionStringSettings settings =
				ConfigurationManager.ConnectionStrings["WF_Persist"];

			SqlWorkflowPersistenceService sqlSvc = new SqlWorkflowPersistenceService(
			   settings.ConnectionString,
			   true, TimeSpan.FromSeconds(10), TimeSpan.FromSeconds(10));

			WorkflowRuntime.AddService(sqlSvc);
		}

		private Stopwatch GetTimerByWorkflow(Guid instanceId)
		{
			if ( !timers .ContainsKey(instanceId))
			{
				timers[instanceId] = new Stopwatch();
			}

			return timers[instanceId];
		}

		private static bool GetUseDB()
		{
			return Convert.ToBoolean( ConfigurationManager.AppSettings["usePersistDB"] );
		}
	}
}