using System;
using System.Workflow.ComponentModel;
using System.Workflow.Runtime;

//////////////////////////////////////////////////////////////////////
// Created by Michael Kennedy
// http://www.michaelckennedy.net
// Copyright Michael Kennedy 2008.
// Part of the HRWorkflow sample application featured in MSDN Magazine.
//

namespace WorkflowCore
{
	public abstract class AsyncActivity : Activity
	{
		protected string QueueName { get; private set; }

		protected AsyncActivity(string queueName)
		{
			this.QueueName = queueName;
		}

		protected override void Initialize(IServiceProvider provider)
		{
			WorkflowQueuingService svc =
				provider.GetService( typeof( WorkflowQueuingService ) ) as WorkflowQueuingService;

			GetQueueInternal( svc );
			base.Initialize( provider );
		}

		protected WorkflowQueue GetQueue(ActivityExecutionContext ctx)
		{
			return GetQueueInternal( ctx.GetService<WorkflowQueuingService>() );
		}

		private WorkflowQueue GetQueueInternal(WorkflowQueuingService svc)
		{
			if (!svc.Exists(QueueName))
			{
				return svc.CreateWorkflowQueue(QueueName, false);
			}

			return svc.GetWorkflowQueue(QueueName);
		}

		private bool hasSubscribed = false;

		protected void SubscribeToItemAvailable(ActivityExecutionContext ctx)
		{
			if ( hasSubscribed )
			{
				return;
			}
			hasSubscribed = true;
			WorkflowQueue queue = GetQueue( ctx );
			queue.QueueItemAvailable += queueItemAvailable;
		}

		private void queueItemAvailable(object sender, QueueEventArgs e)
		{
			ActivityExecutionContext ctx = (ActivityExecutionContext)sender;
			try
			{
				OnQueueItemAvailable(ctx);
			}
			finally
			{
				ctx.CloseActivity();
			}
		}

		protected abstract void OnQueueItemAvailable(ActivityExecutionContext ctx);
	}
}