﻿using System;
using System.Workflow.ComponentModel;
using WorkflowCore.Logging;
using WorkflowCore.Data;

//////////////////////////////////////////////////////////////////////
// Created by Michael Kennedy
// http://www.michaelckennedy.net
// Copyright Michael Kennedy 2008.
// Part of the HRWorkflow sample application featured in MSDN Magazine.
//

namespace WorkflowCore
{
	public partial class ConfirmJobActivity : AsyncActivity
	{
		public const string QUEUE_NAME = "ConfirmJobQueue";

		public ConfirmJobActivity()
			: base( QUEUE_NAME )
		{
			InitializeComponent();
		}

		protected override ActivityExecutionStatus Execute(ActivityExecutionContext ctx)
		{
			TraceDT.Add( "Execution beginning, waiting on queue.", 
				this.WorkflowInstanceId, this.GetType() );

			SubscribeToItemAvailable( ctx );
			return ActivityExecutionStatus.Executing;
		}

		protected override void OnQueueItemAvailable(ActivityExecutionContext ctx)
		{
			EmployeeResponse = GetQueue( ctx ).Dequeue().ToString();
			
			Employee employee = Database.FindEmployee( this.WorkflowInstanceId );
			employee.Accepted = GetAcceptedStatus();

			TraceDT.Add( "Execution completing, queue item available: Response: {0}.", 
				this.WorkflowInstanceId, this.GetType(), EmployeeResponse );
		}

		private bool GetAcceptedStatus()
		{
			return bool.Parse( EmployeeResponse.ToLower().Replace( "yes", "true" ).Replace( "no", "false" ) );
		}

		#region DependencyProperty: public string EmployeeResponse

		public static DependencyProperty EmployeeResponseProperty =
			DependencyProperty.Register( "EmployeeResponse",
										 typeof( string ), typeof( ConfirmJobActivity ) );

		public string EmployeeResponse
		{
			get { return ((string)base.GetValue( EmployeeResponseProperty )); }
			set { base.SetValue( EmployeeResponseProperty, value ); }
		} 
		#endregion
	}
}