﻿using System;
using System.Linq;
using System.Collections.Generic;

//////////////////////////////////////////////////////////////////////
// Created by Michael Kennedy
// http://www.michaelckennedy.net
// Copyright Michael Kennedy 2008.
// Part of the HRWorkflow sample application featured in MSDN Magazine.
//

namespace WorkflowCore.Logging
{
	public class TraceDT
	{
		public static void Add(string text, Guid wfID, Type source, params object[] args)
		{
			string message = string.Format(
				"{0} - {1}: {2}", 
				DateTime.Now, source.Name, string.Format( text, args ));

			TraceEntry entry = new TraceEntry( message );
			
			List<TraceEntry> list;
			if ( wfID == Guid.Empty )
			{
				list = GlobalMessageList;
			}
			else
			{
				list = GetSessionMessageList( wfID );
			}

			list.Add( entry );
		}

		private static List<TraceEntry> GlobalMessageList = new List<TraceEntry>();

		private static Dictionary<Guid, List<TraceEntry>> WfSpecificMessageList = 
			new Dictionary<Guid, List<TraceEntry>>();

		public static string[] GetMessages(bool newestFirst, Guid wfId)
		{
			lock ( GlobalMessageList )
			{
				var messages =
					from entry in GlobalMessageList.Union( GetSessionMessageList(wfId) )
					orderby entry.Created
					select entry.Message;

				if ( newestFirst )
				{
					return messages.Reverse().ToArray();
				}
				else
				{
					return messages.ToArray();
				}
			}
		}

		private static List<TraceEntry> GetSessionMessageList(Guid wfId)
		{
			if ( wfId == Guid.Empty )
			{
				return new List<TraceEntry>();
			}

			if ( WfSpecificMessageList.ContainsKey( wfId ) )
			{
				return WfSpecificMessageList[wfId];
			}

			WfSpecificMessageList[wfId] = new List<TraceEntry>();

			return WfSpecificMessageList[wfId];
		}
	}
}
